from setuptools import setup, find_packages
from pathlib import Path

long_description = (Path(__file__).parent / "README.md").read_text(encoding="utf-8")

setup(
    name='llm_connect',
    version='1.0.1',
    packages=find_packages(),
    install_requires=['click==8.1.7', 'setuptools~=68.2.0', 'transformers~=4.34.0','torch~=2.1.0','accelerate', 'bitsandbytes','colorama'],
    description="LLM Connect API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="CeADAR Connect Group",
    entry_points={
    'console_scripts': [
        'lc = llm_connect_api:cli',
    ],
},
 
)

